@@ -24,7 +24,7 @@
 
 #include <pango/pango-attributes.h>
 #include <pango/pango-context.h>
-#include <pango/pango-glyph.h>
+#include <pango/pango-glyph-item.h>
 #include <pango/pango-tabs.h>
 
 G_BEGIN_DECLS
@@ -32,7 +32,9 @@ G_BEGIN_DECLS
 typedef struct _PangoLayout      PangoLayout;
 typedef struct _PangoLayoutClass PangoLayoutClass;
 typedef struct _PangoLayoutLine  PangoLayoutLine;
-typedef struct _PangoLayoutRun   PangoLayoutRun;
+
+/* For backwards compatiblity */
+typedef PangoGlyphItem PangoLayoutRun;
 
 typedef enum {
   PANGO_ALIGN_LEFT,
@@ -42,7 +44,8 @@ typedef enum {
 
 typedef enum {
   PANGO_WRAP_WORD,
-  PANGO_WRAP_CHAR
+  PANGO_WRAP_CHAR,
+  PANGO_WRAP_WORD_CHAR
 } PangoWrapMode;
 
 struct _PangoLayoutLine
@@ -51,12 +54,8 @@ struct _PangoLayoutLine
   gint         start_index;     /* start of line as byte index into layout->text */
   gint         length;		/* length of line in bytes */
   GSList      *runs;
-};
-
-struct _PangoLayoutRun
-{
-  PangoItem        *item;
-  PangoGlyphString *glyphs;
+  guint        is_paragraph_start : 1;  /* TRUE if this is the first line of the paragraph */ 
+  guint        resolved_dir : 3;  /* Resolved direction of line */
 };
 
 #define PANGO_TYPE_LAYOUT              (pango_layout_get_type ())
@@ -66,7 +65,6 @@ struct _PangoLayoutRun
 #define PANGO_IS_LAYOUT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_LAYOUT))
 #define PANGO_LAYOUT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_LAYOUT, PangoLayoutClass))
 
-
 /* The PangoLayout and PangoLayoutClass structs are private; if you
  * need to create a subclass of these, mail otaylor@redhat.com
  */
@@ -113,6 +111,9 @@ int            pango_layout_get_spacing          (PangoLayout                *la
 void           pango_layout_set_justify          (PangoLayout                *layout,
 						  gboolean                    justify);
 gboolean       pango_layout_get_justify          (PangoLayout                *layout);
+void           pango_layout_set_auto_dir          (PangoLayout                *layout,
+						  gboolean                    auto_dir);
+gboolean       pango_layout_get_auto_dir          (PangoLayout                *layout);
 void           pango_layout_set_alignment        (PangoLayout                *layout,
 						  PangoAlignment              alignment);
 PangoAlignment pango_layout_get_alignment        (PangoLayout                *layout);
@@ -133,10 +134,10 @@ void     pango_layout_get_log_attrs (PangoLayout    *layout,
 				     gint           *n_attrs);
 
 void     pango_layout_index_to_pos         (PangoLayout    *layout,
-					    int             index,
+					    int             index_,
 					    PangoRectangle *pos);
 void     pango_layout_get_cursor_pos       (PangoLayout    *layout,
-					    int             index,
+					    int             index_,
 					    PangoRectangle *strong_pos,
 					    PangoRectangle *weak_pos);
 void     pango_layout_move_cursor_visually (PangoLayout    *layout,
@@ -149,7 +150,7 @@ void     pango_layout_move_cursor_visually (PangoLayout    *layout,
 gboolean pango_layout_xy_to_index          (PangoLayout    *layout,
 					    int             x,
 					    int             y,
-					    int            *index,
+					    int            *index_,
 					    int            *trailing);
 void     pango_layout_get_extents          (PangoLayout    *layout,
 					    PangoRectangle *ink_rect,
@@ -173,10 +174,10 @@ void     pango_layout_line_ref          (PangoLayoutLine  *line);
 void     pango_layout_line_unref        (PangoLayoutLine  *line);
 gboolean pango_layout_line_x_to_index   (PangoLayoutLine  *line,
 					 int               x_pos,
-					 int              *index,
+					 int              *index_,
 					 int              *trailing);
 void     pango_layout_line_index_to_x   (PangoLayoutLine  *line,
-					 int               index,
+					 int               index_,
 					 gboolean          trailing,
 					 int              *x_pos);
 void     pango_layout_line_get_x_ranges (PangoLayoutLine  *line,
@@ -193,6 +194,10 @@ void     pango_layout_line_get_pixel_extents (PangoLayoutLine *layout_line,
 
 typedef struct _PangoLayoutIter PangoLayoutIter;
 
+#define PANGO_TYPE_LAYOUT_ITER         (pango_layout_iter_get_type ())
+
+GType            pango_layout_iter_get_type (void);
+
 PangoLayoutIter *pango_layout_get_iter  (PangoLayout     *layout);
 void             pango_layout_iter_free (PangoLayoutIter *iter);
 
@@ -221,8 +226,8 @@ void pango_layout_iter_get_line_extents    (PangoLayoutIter *iter,
  * assign between-line spacing to the nearest line)
  */
 void pango_layout_iter_get_line_yrange     (PangoLayoutIter *iter,
-                                            int             *y0,
-                                            int             *y1);
+                                            int             *y0_,
+                                            int             *y1_);
 void pango_layout_iter_get_layout_extents  (PangoLayoutIter *iter,
                                             PangoRectangle  *ink_rect,
                                             PangoRectangle  *logical_rect);
