@@ -261,7 +261,7 @@ pango_win32_font_map_for_display (void)
   logfont.lfCharSet = DEFAULT_CHARSET;
   EnumFontFamiliesExA (pango_win32_hdc, &logfont, (FONTENUMPROC) pango_win32_enum_proc, 0, 0);
 
-  fontmap->resolution = PANGO_SCALE / GetDeviceCaps (pango_win32_hdc, LOGPIXELSY) * 72.0;
+  fontmap->resolution = (PANGO_SCALE / (double) GetDeviceCaps (pango_win32_hdc, LOGPIXELSY)) * 72.0;
 
   return PANGO_FONT_MAP (fontmap);
 }
@@ -752,6 +752,7 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
   stretch = PANGO_STRETCH_NORMAL;
 
   font_family = pango_win32_get_font_family (win32fontmap, family_name);
+  g_free (family_name);
 
   tmp_list = font_family->font_entries;
   while (tmp_list)
